@@ -586,9 +586,7 @@ static HRESULT WINAPI IDirectSoundBufferImpl_SetPan(IDirectSoundBuffer8 *iface,
 		return DSERR_INVALIDPARAM;
 	}
 
-	/* You cannot use both pan and 3D controls */
-	if (!(This->dsbd.dwFlags & DSBCAPS_CTRLPAN) ||
-	    (This->dsbd.dwFlags & DSBCAPS_CTRL3D)) {
+	if (!(This->dsbd.dwFlags & DSBCAPS_CTRLPAN)) {
 		WARN("control unavailable\n");
 		return DSERR_CONTROLUNAVAIL;
 	}
@@ -985,38 +983,33 @@ static const IDirectSoundBuffer8Vtbl dsbvt =
 	IDirectSoundBufferImpl_GetObjectInPath
 };
 
-HRESULT IDirectSoundBufferImpl_Create(
-	DirectSoundDevice * device,
-	IDirectSoundBufferImpl **pdsb,
-	LPCDSBUFFERDESC dsbd)
+HRESULT secondarybuffer_create(DirectSoundDevice *device, const DSBUFFERDESC *dsbd,
+        IDirectSoundBuffer **buffer)
 {
 	IDirectSoundBufferImpl *dsb;
 	LPWAVEFORMATEX wfex = dsbd->lpwfxFormat;
 	HRESULT err = DS_OK;
 	DWORD capf = 0;
-	TRACE("(%p,%p,%p)\n",device,pdsb,dsbd);
+
+        TRACE("(%p,%p,%p)\n", device, dsbd, buffer);
 
 	if (dsbd->dwBufferBytes < DSBSIZE_MIN || dsbd->dwBufferBytes > DSBSIZE_MAX) {
 		WARN("invalid parameter: dsbd->dwBufferBytes = %d\n", dsbd->dwBufferBytes);
-		*pdsb = NULL;
 		return DSERR_INVALIDPARAM; /* FIXME: which error? */
 	}
 
 	dsb = HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,sizeof(*dsb));
 
-	if (dsb == 0) {
-		WARN("out of memory\n");
-		*pdsb = NULL;
+        if (!dsb)
 		return DSERR_OUTOFMEMORY;
-	}
 
 	TRACE("Created buffer at %p\n", dsb);
 
-	dsb->ref = 0;
+        dsb->ref = 1;
         dsb->refn = 0;
         dsb->ref3D = 0;
         dsb->refiks = 0;
-	dsb->numIfaces = 0;
+        dsb->numIfaces = 1;
 	dsb->device = device;
 	dsb->IDirectSoundBuffer8_iface.lpVtbl = &dsbvt;
         dsb->IDirectSoundNotify_iface.lpVtbl = &dsnvt;
@@ -1027,9 +1020,8 @@ HRESULT IDirectSoundBufferImpl_Create(
 	CopyMemory(&dsb->dsbd, dsbd, dsbd->dwSize);
 
 	dsb->pwfx = DSOUND_CopyFormat(wfex);
-	if (dsb->pwfx == NULL) {
-		HeapFree(GetProcessHeap(),0,dsb);
-		*pdsb = NULL;
+        if (!dsb->pwfx) {
+                IDirectSoundBuffer8_Release(&dsb->IDirectSoundBuffer8_iface);
 		return DSERR_OUTOFMEMORY;
 	}
 
@@ -1054,22 +1046,16 @@ HRESULT IDirectSoundBufferImpl_Create(
 
 	/* Allocate an empty buffer */
 	dsb->buffer = HeapAlloc(GetProcessHeap(),0,sizeof(*(dsb->buffer)));
-	if (dsb->buffer == NULL) {
-		WARN("out of memory\n");
-		HeapFree(GetProcessHeap(),0,dsb->pwfx);
-		HeapFree(GetProcessHeap(),0,dsb);
-		*pdsb = NULL;
+        if (!dsb->buffer) {
+                IDirectSoundBuffer8_Release(&dsb->IDirectSoundBuffer8_iface);
 		return DSERR_OUTOFMEMORY;
 	}
 
 	/* Allocate system memory for buffer */
 	dsb->buffer->memory = HeapAlloc(GetProcessHeap(),0,dsb->buflen);
-	if (dsb->buffer->memory == NULL) {
+        if (!dsb->buffer->memory) {
 		WARN("out of memory\n");
-		HeapFree(GetProcessHeap(),0,dsb->pwfx);
-		HeapFree(GetProcessHeap(),0,dsb->buffer);
-		HeapFree(GetProcessHeap(),0,dsb);
-		*pdsb = NULL;
+                IDirectSoundBuffer8_Release(&dsb->IDirectSoundBuffer8_iface);
 		return DSERR_OUTOFMEMORY;
 	}
 
@@ -1117,21 +1103,13 @@ HRESULT IDirectSoundBufferImpl_Create(
 
 	RtlInitializeResource(&dsb->lock);
 
-	/* register buffer if not primary */
-	if (!(dsbd->dwFlags & DSBCAPS_PRIMARYBUFFER)) {
-		err = DirectSoundDevice_AddBuffer(device, dsb);
-		if (err != DS_OK) {
-			HeapFree(GetProcessHeap(),0,dsb->buffer->memory);
-			HeapFree(GetProcessHeap(),0,dsb->buffer);
-			RtlDeleteResource(&dsb->lock);
-			HeapFree(GetProcessHeap(),0,dsb->pwfx);
-			HeapFree(GetProcessHeap(),0,dsb);
-			dsb = NULL;
-		}
-	}
+        /* register buffer */
+        err = DirectSoundDevice_AddBuffer(device, dsb);
+        if (err == DS_OK)
+                *buffer = (IDirectSoundBuffer*)&dsb->IDirectSoundBuffer8_iface;
+        else
+                IDirectSoundBuffer8_Release(&dsb->IDirectSoundBuffer8_iface);
 
-        IDirectSoundBuffer8_AddRef(&dsb->IDirectSoundBuffer8_iface);
-	*pdsb = dsb;
 	return err;
 }
 
