@@ -211,15 +211,18 @@ static BOOL put_console_into_raw_mode(int fd)
 static BOOL restore_console_mode(HANDLE hin)
 {
     int         fd;
-    BOOL        ret;
+    BOOL        ret = TRUE;
+
+    if (S_termios_raw)
+    {
+        if ((fd = get_console_bare_fd(hin)) == -1) return FALSE;
+        ret = tcsetattr(fd, TCSANOW, &S_termios) >= 0;
+        close(fd);
+    }
+
+    if (RtlGetCurrentPeb()->ProcessParameters->ConsoleHandle == KERNEL32_CONSOLE_SHELL)
+        TERM_Exit();
 
-    if (!S_termios_raw ||
-        RtlGetCurrentPeb()->ProcessParameters->ConsoleHandle != KERNEL32_CONSOLE_SHELL)
-        return TRUE;
-    if ((fd = get_console_bare_fd(hin)) == -1) return FALSE;
-    ret = tcsetattr(fd, TCSANOW, &S_termios) >= 0;
-    close(fd);
-    TERM_Exit();
     return ret;
 }
 
@@ -3234,6 +3237,20 @@ BOOL WINAPI SetConsoleIcon(HICON icon)
     return FALSE;
 }
 
+DWORD WINAPI GetNumberOfConsoleFonts(void)
+{
+    return 1;
+}
+
+
+BOOL WINAPI SetConsoleKeyShortcuts(BOOL set, BYTE keys, VOID *a, DWORD b)
+{
+    FIXME(": (%u %u %p %u) stub!\n", set, keys, a, b);
+    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+    return FALSE;
+}
+
+
 BOOL WINAPI GetCurrentConsoleFont(HANDLE hConsole, BOOL maxwindow, LPCONSOLE_FONT_INFO fontinfo)
 {
     BOOL ret;
@@ -3260,32 +3277,47 @@ BOOL WINAPI GetCurrentConsoleFont(HANDLE hConsole, BOOL maxwindow, LPCONSOLE_FON
     return ret;
 }
 
+static COORD get_console_font_size(HANDLE hConsole, DWORD index)
+{
+    COORD c = {0,0};
+
+    if (index >= GetNumberOfConsoleFonts())
+    {
+        SetLastError(ERROR_INVALID_PARAMETER);
+        return c;
+    }
+
+    SERVER_START_REQ(get_console_output_info)
+    {
+        req->handle = console_handle_unmap(hConsole);
+        if (!wine_server_call_err(req))
+        {
+            c.X = reply->font_width;
+            c.Y = reply->font_height;
+        }
+    }
+    SERVER_END_REQ;
+    return c;
+}
+
 #ifdef __i386__
 #undef GetConsoleFontSize
-DWORD WINAPI GetConsoleFontSize(HANDLE hConsole, DWORD font)
+DWORD WINAPI GetConsoleFontSize(HANDLE hConsole, DWORD index)
 {
     union {
         COORD c;
         DWORD w;
     } x;
 
-    FIXME(": (%p, %d) stub!\n", hConsole, font);
-    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
-
-    x.c.X = 0;
-    x.c.Y = 0;
+    x.c = get_console_font_size(hConsole, index);
     return x.w;
 }
 #endif /* defined(__i386__) */
 
 
 #ifndef __i386__
-COORD WINAPI GetConsoleFontSize(HANDLE hConsole, DWORD font)
+COORD WINAPI GetConsoleFontSize(HANDLE hConsole, DWORD index)
 {
-    COORD c;
-    c.X = 80;
-    c.Y = 24;
-     FIXME(": (%p, %d) stub!\n", hConsole, font);
-    return c;
+    return get_console_font_size(hConsole, index);
 }
-#endif /* defined(__i386__) */
+#endif /* !defined(__i386__) */
