@@ -129,6 +129,9 @@ static enum file_type (__cdecl *p_tr2_sys__Stat)(char const*, int *);
 static enum file_type (__cdecl *p_tr2_sys__Lstat)(char const*, int *);
 static __int64 (__cdecl *p_tr2_sys__Last_write_time)(char const*);
 static void (__cdecl *p_tr2_sys__Last_write_time_set)(char const*, __int64);
+static void* (__cdecl *p_tr2_sys__Open_dir)(char*, char const*, int *, enum file_type*);
+static char* (__cdecl *p_tr2_sys__Read_dir)(char*, void*, enum file_type*);
+static void (__cdecl *p_tr2_sys__Close_dir)(void*);
 
 /* thrd */
 typedef struct
@@ -137,8 +140,47 @@ typedef struct
     DWORD  id;
 } _Thrd_t;
 
+#define TIMEDELTA 250  /* 250 ms uncertainty allowed */
+
+typedef int (__cdecl *_Thrd_start_t)(void*);
+
 static int (__cdecl *p__Thrd_equal)(_Thrd_t, _Thrd_t);
 static int (__cdecl *p__Thrd_lt)(_Thrd_t, _Thrd_t);
+static void (__cdecl *p__Thrd_sleep)(const xtime*);
+static _Thrd_t (__cdecl *p__Thrd_current)(void);
+static int (__cdecl *p__Thrd_create)(_Thrd_t*, _Thrd_start_t, void*);
+static int (__cdecl *p__Thrd_join)(_Thrd_t, int*);
+
+#ifdef __i386__
+static ULONGLONG (__cdecl *p_i386_Thrd_current)(void);
+static _Thrd_t __cdecl i386_Thrd_current(void)
+{
+    union {
+        _Thrd_t thr;
+        ULONGLONG ull;
+    } r;
+    r.ull = p_i386_Thrd_current();
+    return r.thr;
+}
+#endif
+
+/* mtx */
+typedef void *_Mtx_t;
+static int (__cdecl *p__Mtx_init)(_Mtx_t*, int);
+static void (__cdecl *p__Mtx_destroy)(_Mtx_t*);
+static int (__cdecl *p__Mtx_lock)(_Mtx_t*);
+static int (__cdecl *p__Mtx_unlock)(_Mtx_t*);
+
+/* cnd */
+typedef void *_Cnd_t;
+
+static int (__cdecl *p__Cnd_init)(_Cnd_t*);
+static void (__cdecl *p__Cnd_destroy)(_Cnd_t*);
+static int (__cdecl *p__Cnd_wait)(_Cnd_t*, _Mtx_t*);
+static int (__cdecl *p__Cnd_timedwait)(_Cnd_t*, _Mtx_t*, const xtime*);
+static int (__cdecl *p__Cnd_broadcast)(_Cnd_t*);
+static int (__cdecl *p__Cnd_signal)(_Cnd_t*);
+
 
 static HMODULE msvcp;
 #define SETNOFAIL(x,y) x = (void*)GetProcAddress(msvcp,y)
@@ -217,6 +259,14 @@ static BOOL init(void)
                 "?_Last_write_time@sys@tr2@std@@YA_JPEBD@Z");
         SET(p_tr2_sys__Last_write_time_set,
                 "?_Last_write_time@sys@tr2@std@@YAXPEBD_J@Z");
+        SET(p_tr2_sys__Open_dir,
+                "?_Open_dir@sys@tr2@std@@YAPEAXAEAY0BAE@DPEBDAEAHAEAW4file_type@123@@Z");
+        SET(p_tr2_sys__Read_dir,
+                "?_Read_dir@sys@tr2@std@@YAPEADAEAY0BAE@DPEAXAEAW4file_type@123@@Z");
+        SET(p_tr2_sys__Close_dir,
+                "?_Close_dir@sys@tr2@std@@YAXPEAX@Z");
+        SET(p__Thrd_current,
+                "_Thrd_current");
     } else {
         SET(p_tr2_sys__File_size,
                 "?_File_size@sys@tr2@std@@YA_KPBD@Z");
@@ -262,11 +312,53 @@ static BOOL init(void)
                 "?_Last_write_time@sys@tr2@std@@YA_JPBD@Z");
         SET(p_tr2_sys__Last_write_time_set,
                 "?_Last_write_time@sys@tr2@std@@YAXPBD_J@Z");
+        SET(p_tr2_sys__Open_dir,
+                "?_Open_dir@sys@tr2@std@@YAPAXAAY0BAE@DPBDAAHAAW4file_type@123@@Z");
+        SET(p_tr2_sys__Read_dir,
+                "?_Read_dir@sys@tr2@std@@YAPADAAY0BAE@DPAXAAW4file_type@123@@Z");
+        SET(p_tr2_sys__Close_dir,
+                "?_Close_dir@sys@tr2@std@@YAXPAX@Z");
+#ifdef __i386__
+        SET(p_i386_Thrd_current,
+                "_Thrd_current");
+        p__Thrd_current = i386_Thrd_current;
+#else
+        SET(p__Thrd_current,
+                "_Thrd_current");
+#endif
     }
     SET(p__Thrd_equal,
             "_Thrd_equal");
     SET(p__Thrd_lt,
             "_Thrd_lt");
+    SET(p__Thrd_sleep,
+            "_Thrd_sleep");
+    SET(p__Thrd_create,
+            "_Thrd_create");
+    SET(p__Thrd_join,
+            "_Thrd_join");
+
+    SET(p__Mtx_init,
+            "_Mtx_init");
+    SET(p__Mtx_destroy,
+            "_Mtx_destroy");
+    SET(p__Mtx_lock,
+            "_Mtx_lock");
+    SET(p__Mtx_unlock,
+            "_Mtx_unlock");
+
+    SET(p__Cnd_init,
+            "_Cnd_init");
+    SET(p__Cnd_destroy,
+            "_Cnd_destroy");
+    SET(p__Cnd_wait,
+            "_Cnd_wait");
+    SET(p__Cnd_timedwait,
+            "_Cnd_timedwait");
+    SET(p__Cnd_broadcast,
+            "_Cnd_broadcast");
+    SET(p__Cnd_signal,
+            "_Cnd_signal");
 
     msvcr = GetModuleHandleA("msvcr120.dll");
     p_setlocale = (void*)GetProcAddress(msvcr, "setlocale");
@@ -1134,9 +1226,110 @@ static void test_tr2_sys__Last_write_time(void)
     ok(ret == 1, "test_tr2_sys__Remove_dir(): expect 1 got %d\n", ret);
 }
 
+static void test_tr2_sys__dir_operation(void)
+{
+    char *file_name, first_file_name[MAX_PATH], dest[MAX_PATH], longer_path[MAX_PATH];
+    HANDLE file, result_handle;
+    enum file_type type;
+    int err, num_of_f1 = 0, num_of_f2 = 0, num_of_sub_dir = 0, num_of_other_files = 0;
+
+    CreateDirectoryA("tr2_test_dir", NULL);
+    file = CreateFileA("tr2_test_dir/f1", 0, 0, NULL, CREATE_ALWAYS, 0, NULL);
+    ok(file != INVALID_HANDLE_VALUE, "create file failed: INVALID_HANDLE_VALUE\n");
+    CloseHandle(file);
+    file = CreateFileA("tr2_test_dir/f2", 0, 0, NULL, CREATE_ALWAYS, 0, NULL);
+    ok(file != INVALID_HANDLE_VALUE, "create file failed: INVALID_HANDLE_VALUE\n");
+    CloseHandle(file);
+    CreateDirectoryA("tr2_test_dir/sub_dir", NULL);
+    file = CreateFileA("tr2_test_dir/sub_dir/sub_f1", 0, 0, NULL, CREATE_ALWAYS, 0, NULL);
+    ok(file != INVALID_HANDLE_VALUE, "create file failed: INVALID_HANDLE_VALUE\n");
+    CloseHandle(file);
+
+    memset(longer_path, 0, MAX_PATH);
+    GetCurrentDirectoryA(MAX_PATH, longer_path);
+    strcat(longer_path, "\\tr2_test_dir\\");
+    while(lstrlenA(longer_path) < MAX_PATH-1)
+        strcat(longer_path, "s");
+    ok(lstrlenA(longer_path) == MAX_PATH-1, "tr2_sys__Open_dir(): expect MAX_PATH, got %d\n", lstrlenA(longer_path));
+    memset(first_file_name, 0, MAX_PATH);
+    type = err =  0xdeadbeef;
+    result_handle = NULL;
+    result_handle = p_tr2_sys__Open_dir(first_file_name, longer_path, &err, &type);
+    ok(result_handle == NULL, "tr2_sys__Open_dir(): expect NULL, got %p\n", result_handle);
+    ok(!*first_file_name, "tr2_sys__Open_dir(): expect: 0, got %s\n", first_file_name);
+    ok(err == ERROR_BAD_PATHNAME, "tr2_sys__Open_dir(): expect: ERROR_BAD_PATHNAME, got %d\n", err);
+    ok((int)type == 0xdeadbeef, "tr2_sys__Open_dir(): expect 0xdeadbeef, got %d\n", type);
+
+    memset(first_file_name, 0, MAX_PATH);
+    memset(dest, 0, MAX_PATH);
+    err = type = 0xdeadbeef;
+    result_handle = NULL;
+    result_handle = p_tr2_sys__Open_dir(first_file_name, "tr2_test_dir", &err, &type);
+    ok(result_handle != NULL, "tr2_sys__Open_dir(): expect: not NULL, got %p\n", result_handle);
+    ok(err == ERROR_SUCCESS, "tr2_sys__Open_dir(): expect: ERROR_SUCCESS, got %d\n", err);
+    file_name = first_file_name;
+    while(*file_name) {
+        if (!strcmp(file_name, "f1")) {
+            ++num_of_f1;
+            ok(type == regular_file, "expect regular_file, got %d\n", type);
+        }else if(!strcmp(file_name, "f2")) {
+            ++num_of_f2;
+            ok(type == regular_file, "expect regular_file, got %d\n", type);
+        }else if(!strcmp(file_name, "sub_dir")) {
+            ++num_of_sub_dir;
+            ok(type == directory_file, "expect directory_file, got %d\n", type);
+        }else {
+            ++num_of_other_files;
+        }
+        file_name = p_tr2_sys__Read_dir(dest, result_handle, &type);
+    }
+    p_tr2_sys__Close_dir(result_handle);
+    ok(result_handle != NULL, "tr2_sys__Open_dir(): expect: not NULL, got %p\n", result_handle);
+    ok(num_of_f1 == 1, "found f1 %d times\n", num_of_f1);
+    ok(num_of_f2 == 1, "found f2 %d times\n", num_of_f2);
+    ok(num_of_sub_dir == 1, "found sub_dir %d times\n", num_of_sub_dir);
+    ok(num_of_other_files == 0, "found %d other files\n", num_of_other_files);
+
+    memset(first_file_name, 0, MAX_PATH);
+    err = type = 0xdeadbeef;
+    result_handle = file;
+    result_handle = p_tr2_sys__Open_dir(first_file_name, "not_exist", &err, &type);
+    ok(result_handle == NULL, "tr2_sys__Open_dir(): expect: NULL, got %p\n", result_handle);
+    todo_wine ok(err == ERROR_BAD_PATHNAME, "tr2_sys__Open_dir(): expect: ERROR_BAD_PATHNAME, got %d\n", err);
+    ok((int)type == 0xdeadbeef, "tr2_sys__Open_dir(): expect: 0xdeadbeef, got %d\n", type);
+    ok(!*first_file_name, "tr2_sys__Open_dir(): expect: 0, got %s\n", first_file_name);
+
+    CreateDirectoryA("empty_dir", NULL);
+    memset(first_file_name, 0, MAX_PATH);
+    err = type = 0xdeadbeef;
+    result_handle = file;
+    result_handle = p_tr2_sys__Open_dir(first_file_name, "empty_dir", &err, &type);
+    ok(result_handle == NULL, "tr2_sys__Open_dir(): expect: NULL, got %p\n", result_handle);
+    ok(err == ERROR_SUCCESS, "tr2_sys__Open_dir(): expect: ERROR_SUCCESS, got %d\n", err);
+    ok(type == status_unknown, "tr2_sys__Open_dir(): expect: status_unknown, got %d\n", type);
+    ok(!*first_file_name, "tr2_sys__Open_dir(): expect: 0, got %s\n", first_file_name);
+    p_tr2_sys__Close_dir(result_handle);
+    ok(result_handle == NULL, "tr2_sys__Open_dir(): expect: NULL, got %p\n", result_handle);
+
+    ok(RemoveDirectoryA("empty_dir"), "expect empty_dir to exist\n");
+    ok(DeleteFileA("tr2_test_dir/sub_dir/sub_f1"), "expect tr2_test_dir/sub_dir/sub_f1 to exist\n");
+    ok(RemoveDirectoryA("tr2_test_dir/sub_dir"), "expect tr2_test_dir/sub_dir to exist\n");
+    ok(DeleteFileA("tr2_test_dir/f1"), "expect tr2_test_dir/f1 to exist\n");
+    ok(DeleteFileA("tr2_test_dir/f2"), "expect tr2_test_dir/f2 to exist\n");
+    ok(RemoveDirectoryA("tr2_test_dir"), "expect tr2_test_dir to exist\n");
+}
+
+static int __cdecl thrd_thread(void *arg)
+{
+    _Thrd_t *thr = arg;
+
+    *thr = p__Thrd_current();
+    return 0x42;
+}
+
 static void test_thrd(void)
 {
-    int ret, i;
+    int ret, i, r;
     struct test {
         _Thrd_t a;
         _Thrd_t b;
@@ -1144,6 +1337,9 @@ static void test_thrd(void)
     };
     const HANDLE hnd1 = (HANDLE)0xcccccccc;
     const HANDLE hnd2 = (HANDLE)0xdeadbeef;
+    xtime xt, before, after;
+    MSVCRT_long diff;
+    _Thrd_t ta, tb;
 
     struct test testeq[] = {
         { {0,    0}, {0,    0}, 1 },
@@ -1173,6 +1369,184 @@ static void test_thrd(void)
         ok(ret == testlt[i].r, "(%p %u) < (%p %u) expected %d, got %d\n",
             testlt[i].a.hnd, testlt[i].a.id, testlt[i].b.hnd, testlt[i].b.id, testlt[i].r, ret);
     }
+
+    /* test for sleep */
+    if (0) /* crash on Windows */
+        p__Thrd_sleep(NULL);
+    p_xtime_get(&xt, 1);
+    xt.sec += 2;
+    p_xtime_get(&before, 1);
+    p__Thrd_sleep(&xt);
+    p_xtime_get(&after, 1);
+    diff = p__Xtime_diff_to_millis2(&after, &before);
+    ok(diff > 2000 - TIMEDELTA, "got %d\n", diff);
+
+    /* test for current */
+    ta = p__Thrd_current();
+    tb = p__Thrd_current();
+    ok(ta.id == tb.id, "got a %d b %d\n", ta.id, tb.id);
+    ok(ta.id == GetCurrentThreadId(), "expected %d, got %d\n", GetCurrentThreadId(), ta.id);
+    /* these can be different if new threads are created at same time */
+    ok(ta.hnd == tb.hnd, "got a %p b %p\n", ta.hnd, tb.hnd);
+    ok(!CloseHandle(ta.hnd), "handle %p not closed\n", ta.hnd);
+    ok(!CloseHandle(tb.hnd), "handle %p not closed\n", tb.hnd);
+
+    /* test for create/join */
+    if (0) /* crash on Windows */
+    {
+        p__Thrd_create(NULL, thrd_thread, NULL);
+        p__Thrd_create(&ta, NULL, NULL);
+    }
+    r = -1;
+    ret = p__Thrd_create(&ta, thrd_thread, (void*)&tb);
+    ok(!ret, "failed to create thread, got %d\n", ret);
+    ret = p__Thrd_join(ta, &r);
+    ok(!ret, "failed to join thread, got %d\n", ret);
+    ok(ta.id == tb.id, "expected %d, got %d\n", ta.id, tb.id);
+    ok(ta.hnd != tb.hnd, "same handles, got %p\n", ta.hnd);
+    ok(r == 0x42, "expected 0x42, got %d\n", r);
+    ok(!CloseHandle(ta.hnd), "handle %p not closed\n", ta.hnd);
+}
+
+#define NUM_THREADS 10
+struct cndmtx
+{
+    HANDLE initialized;
+    int started;
+    int thread_no;
+
+    _Cnd_t cnd;
+    _Mtx_t mtx;
+    BOOL timed_wait;
+};
+
+static int __cdecl cnd_wait_thread(void *arg)
+{
+    struct cndmtx *cm = arg;
+    int r;
+
+    p__Mtx_lock(&cm->mtx);
+
+    if(InterlockedIncrement(&cm->started) == cm->thread_no)
+        SetEvent(cm->initialized);
+
+    if(cm->timed_wait) {
+        xtime xt;
+
+        p_xtime_get(&xt, 1);
+        xt.sec += 2;
+        r = p__Cnd_timedwait(&cm->cnd, &cm->mtx, &xt);
+        ok(!r, "timed wait failed\n");
+    } else {
+        r = p__Cnd_wait(&cm->cnd, &cm->mtx);
+        ok(!r, "wait failed\n");
+    }
+
+    p__Mtx_unlock(&cm->mtx);
+    return 0;
+}
+
+static void test_cnd(void)
+{
+    _Thrd_t threads[NUM_THREADS];
+    xtime xt, before, after;
+    MSVCRT_long diff;
+    struct cndmtx cm;
+    _Cnd_t cnd;
+    _Mtx_t mtx;
+    int r, i;
+
+    r = p__Cnd_init(&cnd);
+    ok(!r, "failed to init cnd\n");
+
+    r = p__Mtx_init(&mtx, 0);
+    ok(!r, "failed to init mtx\n");
+
+    if (0) /* crash on Windows */
+    {
+        p__Cnd_init(NULL);
+        p__Cnd_wait(NULL, &mtx);
+        p__Cnd_wait(&cnd, NULL);
+        p__Cnd_timedwait(NULL, &mtx, &xt);
+        p__Cnd_timedwait(&cnd, &mtx, &xt);
+    }
+    p__Cnd_destroy(NULL);
+
+    /* test _Cnd_signal/_Cnd_wait */
+    cm.initialized = CreateEventW(NULL, FALSE, FALSE, NULL);
+    cm.started = 0;
+    cm.thread_no = 1;
+    cm.cnd = cnd;
+    cm.mtx = mtx;
+    cm.timed_wait = FALSE;
+    p__Thrd_create(&threads[0], cnd_wait_thread, (void*)&cm);
+
+    WaitForSingleObject(cm.initialized, INFINITE);
+    p__Mtx_lock(&mtx);
+    p__Mtx_unlock(&mtx);
+
+    r = p__Cnd_signal(&cm.cnd);
+    ok(!r, "failed to signal\n");
+    p__Thrd_join(threads[0], NULL);
+
+    /* test _Cnd_timedwait time out */
+    p__Mtx_lock(&mtx);
+    p_xtime_get(&before, 1);
+    xt = before;
+    xt.sec += 1;
+    r = p__Cnd_timedwait(&cnd, &mtx, &xt);
+    p_xtime_get(&after, 1);
+    p__Mtx_unlock(&mtx);
+
+    diff = p__Xtime_diff_to_millis2(&after, &before);
+    ok(r == 2, "should have timed out\n");
+    ok(diff > 1000 - TIMEDELTA, "got %d\n", diff);
+
+    /* test _Cnd_timedwait */
+    cm.started = 0;
+    cm.timed_wait = TRUE;
+    p__Thrd_create(&threads[0], cnd_wait_thread, (void*)&cm);
+
+    WaitForSingleObject(cm.initialized, INFINITE);
+    p__Mtx_lock(&mtx);
+    p__Mtx_unlock(&mtx);
+
+    r = p__Cnd_signal(&cm.cnd);
+    ok(!r, "failed to signal\n");
+    p__Thrd_join(threads[0], NULL);
+
+    /* test _Cnd_broadcast */
+    cm.started = 0;
+    cm.thread_no = NUM_THREADS;
+    cm.timed_wait = FALSE;
+
+    for(i = 0; i < cm.thread_no; i++)
+        p__Thrd_create(&threads[i], cnd_wait_thread, (void*)&cm);
+
+    WaitForSingleObject(cm.initialized, INFINITE);
+    p__Mtx_lock(&mtx);
+    p__Mtx_unlock(&mtx);
+
+    r = p__Cnd_broadcast(&cnd);
+    ok(!r, "failed to broadcast\n");
+    for(i = 0; i < cm.thread_no; i++)
+        p__Thrd_join(threads[i], NULL);
+
+    /* test broadcast with _Cnd_destroy */
+    cm.started = 0;
+    for(i = 0; i < cm.thread_no; i++)
+        p__Thrd_create(&threads[i], cnd_wait_thread, (void*)&cm);
+
+    WaitForSingleObject(cm.initialized, INFINITE);
+    p__Mtx_lock(&mtx);
+    p__Mtx_unlock(&mtx);
+
+    p__Cnd_destroy(&cnd);
+    for(i = 0; i < cm.thread_no; i++)
+        p__Thrd_join(threads[i], NULL);
+
+    p__Mtx_destroy(&mtx);
+    CloseHandle(cm.initialized);
 }
 
 START_TEST(msvcp120)
@@ -1198,8 +1572,10 @@ START_TEST(msvcp120)
     test_tr2_sys__Statvfs();
     test_tr2_sys__Stat();
     test_tr2_sys__Last_write_time();
+    test_tr2_sys__dir_operation();
 
     test_thrd();
+    test_cnd();
 
     FreeLibrary(msvcp);
 }
