@@ -288,7 +288,7 @@ static void test_query_process(void)
     /* test ReturnLength */
     ReturnLength = 0;
     status = pNtQuerySystemInformation(SystemProcessInformation, NULL, 0, &ReturnLength);
-    ok( status == STATUS_INFO_LENGTH_MISMATCH, "Expected STATUS_LENGTH_MISMATCH got %08x\n", status);
+    ok( status == STATUS_INFO_LENGTH_MISMATCH, "Expected STATUS_INFO_LENGTH_MISMATCH got %08x\n", status);
     ok( ReturnLength > 0 || broken(ReturnLength == 0) /* NT4, Win2K */,
         "Expected a ReturnLength to show the needed length\n");
 
@@ -474,30 +474,62 @@ static void test_query_module(void)
 static void test_query_handle(void)
 {
     NTSTATUS status;
-    ULONG ReturnLength;
+    ULONG ExpectedLength, ReturnLength;
     ULONG SystemInformationLength = sizeof(SYSTEM_HANDLE_INFORMATION);
     SYSTEM_HANDLE_INFORMATION* shi = HeapAlloc(GetProcessHeap(), 0, SystemInformationLength);
+    HANDLE EventHandle;
+    BOOL found;
+    INT i;
+
+    EventHandle = CreateEventA(NULL, FALSE, FALSE, NULL);
+    ok( EventHandle != NULL, "CreateEventA failed %u\n", GetLastError() );
 
     /* Request the needed length : a SystemInformationLength greater than one struct sets ReturnLength */
+    ReturnLength = 0xdeadbeef;
     status = pNtQuerySystemInformation(SystemHandleInformation, shi, SystemInformationLength, &ReturnLength);
     todo_wine ok( status == STATUS_INFO_LENGTH_MISMATCH, "Expected STATUS_INFO_LENGTH_MISMATCH, got %08x\n", status);
+    ok( ReturnLength != 0xdeadbeef, "Expected valid ReturnLength\n" );
 
     SystemInformationLength = ReturnLength;
     shi = HeapReAlloc(GetProcessHeap(), 0, shi , SystemInformationLength);
-    status = pNtQuerySystemInformation(SystemHandleInformation, shi, SystemInformationLength, &ReturnLength);
-    if (status != STATUS_INFO_LENGTH_MISMATCH) /* vista */
-    {
-        ok( status == STATUS_SUCCESS,
-            "Expected STATUS_SUCCESS, got %08x\n", status);
 
-        /* Check if we have some return values */
-        trace("Number of Handles : %d\n", shi->Count);
-        todo_wine
-        {
-            /* our implementation is a stub for now */
-            ok( shi->Count > 1, "Expected more than 1 handles, got (%d)\n", shi->Count);
-        }
+    ReturnLength = 0xdeadbeef;
+    status = pNtQuerySystemInformation(SystemHandleInformation, shi, SystemInformationLength, &ReturnLength);
+    while (status == STATUS_INFO_LENGTH_MISMATCH) /* Vista / 2008 */
+    {
+        SystemInformationLength *= 2;
+        shi = HeapReAlloc(GetProcessHeap(), 0, shi, SystemInformationLength);
+        status = pNtQuerySystemInformation(SystemHandleInformation, shi, SystemInformationLength, &ReturnLength);
     }
+    ok( status == STATUS_SUCCESS, "Expected STATUS_SUCCESS, got %08x\n", status );
+    ExpectedLength = FIELD_OFFSET(SYSTEM_HANDLE_INFORMATION, Handle[shi->Count]);
+    todo_wine ok( ReturnLength == ExpectedLength || broken(ReturnLength == ExpectedLength - sizeof(DWORD)), /* Vista / 2008 */
+                  "Expected length %u, got %u\n", ExpectedLength, ReturnLength );
+    todo_wine ok( shi->Count > 1, "Expected more than 1 handle, got %u\n", shi->Count );
+    for (i = 0, found = FALSE; i < shi->Count && !found; i++)
+        found = (shi->Handle[i].OwnerPid == GetCurrentProcessId()) &&
+                ((HANDLE)(ULONG_PTR)shi->Handle[i].HandleValue == EventHandle);
+    todo_wine ok( found, "Expected to find event handle in handle list\n" );
+
+    CloseHandle(EventHandle);
+
+    ReturnLength = 0xdeadbeef;
+    status = pNtQuerySystemInformation(SystemHandleInformation, shi, SystemInformationLength, &ReturnLength);
+    while (status == STATUS_INFO_LENGTH_MISMATCH) /* Vista / 2008 */
+    {
+        SystemInformationLength *= 2;
+        shi = HeapReAlloc(GetProcessHeap(), 0, shi, SystemInformationLength);
+        status = pNtQuerySystemInformation(SystemHandleInformation, shi, SystemInformationLength, &ReturnLength);
+    }
+    ok( status == STATUS_SUCCESS, "Expected STATUS_SUCCESS, got %08x\n", status );
+    for (i = 0, found = FALSE; i < shi->Count && !found; i++)
+        found = (shi->Handle[i].OwnerPid == GetCurrentProcessId()) &&
+                ((HANDLE)(ULONG_PTR)shi->Handle[i].HandleValue == EventHandle);
+    ok( !found, "Unexpectedly found event handle in handle list\n" );
+
+    status = pNtQuerySystemInformation(SystemHandleInformation, NULL, SystemInformationLength, &ReturnLength);
+    ok( status == STATUS_ACCESS_VIOLATION, "Expected STATUS_ACCESS_VIOLATION, got %08x\n", status );
+
     HeapFree( GetProcessHeap(), 0, shi);
 }
 
@@ -1050,7 +1082,7 @@ static void test_query_process_debug_port(int argc, char **argv)
 
     status = pNtQueryInformationProcess(NULL, ProcessDebugPort,
             &debug_port, sizeof(debug_port), NULL);
-    ok(status == STATUS_INVALID_HANDLE, "Expected STATUS_ACCESS_VIOLATION, got %#x.\n", status);
+    ok(status == STATUS_INVALID_HANDLE, "Expected STATUS_INVALID_HANDLE, got %#x.\n", status);
 
     status = pNtQueryInformationProcess(GetCurrentProcess(), ProcessDebugPort,
             &debug_port, sizeof(debug_port) - 1, NULL);
@@ -1273,27 +1305,27 @@ static void test_query_process_debug_object_handle(int argc, char **argv)
 
 static void test_query_process_debug_flags(int argc, char **argv)
 {
+    static const DWORD test_flags[] = { DEBUG_PROCESS,
+                                        DEBUG_ONLY_THIS_PROCESS,
+                                        DEBUG_PROCESS | DEBUG_ONLY_THIS_PROCESS,
+                                        CREATE_SUSPENDED };
     DWORD debug_flags = 0xdeadbeef;
     char cmdline[MAX_PATH];
     PROCESS_INFORMATION pi;
     STARTUPINFOA si = { 0 };
     NTSTATUS status;
+    DEBUG_EVENT ev;
+    DWORD result;
     BOOL ret;
+    int i, j;
 
-    sprintf(cmdline, "%s %s %s", argv[0], argv[1], "debuggee");
+    /* test invalid arguments */
+    status = pNtQueryInformationProcess(NULL, ProcessDebugFlags, NULL, 0, NULL);
+    ok(status == STATUS_INFO_LENGTH_MISMATCH || broken(status == STATUS_INVALID_INFO_CLASS) /* WOW64 */,
+            "Expected STATUS_INFO_LENGTH_MISMATCH, got %#x.\n", status);
 
-    si.cb = sizeof(si);
-    ret = CreateProcessA(NULL, cmdline, NULL, NULL, FALSE, DEBUG_PROCESS | DEBUG_ONLY_THIS_PROCESS, NULL, NULL, &si, &pi);
-    ok(ret, "CreateProcess failed, last error %#x.\n", GetLastError());
-    if (!ret) return;
-
-    status = pNtQueryInformationProcess(NULL, ProcessDebugFlags,
-            NULL, 0, NULL);
-    ok(status == STATUS_INFO_LENGTH_MISMATCH || broken(status == STATUS_INVALID_INFO_CLASS) /* NT4 */, "Expected STATUS_INFO_LENGTH_MISMATCH, got %#x.\n", status);
-
-    status = pNtQueryInformationProcess(NULL, ProcessDebugFlags,
-            NULL, sizeof(debug_flags), NULL);
-    ok(status == STATUS_INVALID_HANDLE || status == STATUS_ACCESS_VIOLATION || broken(status == STATUS_INVALID_INFO_CLASS) /* W7PROX64 (32-bit) */,
+    status = pNtQueryInformationProcess(NULL, ProcessDebugFlags, NULL, sizeof(debug_flags), NULL);
+    ok(status == STATUS_INVALID_HANDLE || status == STATUS_ACCESS_VIOLATION || broken(status == STATUS_INVALID_INFO_CLASS) /* WOW64 */,
             "Expected STATUS_INVALID_HANDLE, got %#x.\n", status);
 
     status = pNtQueryInformationProcess(GetCurrentProcess(), ProcessDebugFlags,
@@ -1302,45 +1334,123 @@ static void test_query_process_debug_flags(int argc, char **argv)
 
     status = pNtQueryInformationProcess(NULL, ProcessDebugFlags,
             &debug_flags, sizeof(debug_flags), NULL);
-    ok(status == STATUS_INVALID_HANDLE || broken(status == STATUS_INVALID_INFO_CLASS) /* NT4 */, "Expected STATUS_ACCESS_VIOLATION, got %#x.\n", status);
+    ok(status == STATUS_INVALID_HANDLE || broken(status == STATUS_INVALID_INFO_CLASS) /* WOW64 */,
+            "Expected STATUS_INVALID_HANDLE, got %#x.\n", status);
 
     status = pNtQueryInformationProcess(GetCurrentProcess(), ProcessDebugFlags,
             &debug_flags, sizeof(debug_flags) - 1, NULL);
-    ok(status == STATUS_INFO_LENGTH_MISMATCH || broken(status == STATUS_INVALID_INFO_CLASS) /* NT4 */, "Expected STATUS_INFO_LENGTH_MISMATCH, got %#x.\n", status);
+    ok(status == STATUS_INFO_LENGTH_MISMATCH, "Expected STATUS_INFO_LENGTH_MISMATCH, got %#x.\n", status);
 
     status = pNtQueryInformationProcess(GetCurrentProcess(), ProcessDebugFlags,
             &debug_flags, sizeof(debug_flags) + 1, NULL);
-    ok(status == STATUS_INFO_LENGTH_MISMATCH || broken(status == STATUS_INVALID_INFO_CLASS) /* NT4 */, "Expected STATUS_INFO_LENGTH_MISMATCH, got %#x.\n", status);
+    ok(status == STATUS_INFO_LENGTH_MISMATCH, "Expected STATUS_INFO_LENGTH_MISMATCH, got %#x.\n", status);
 
+    /* test ProcessDebugFlags of current process */
     status = pNtQueryInformationProcess(GetCurrentProcess(), ProcessDebugFlags,
             &debug_flags, sizeof(debug_flags), NULL);
-    ok(!status || broken(status == STATUS_INVALID_INFO_CLASS) /* NT4 */, "NtQueryInformationProcess failed, status %#x.\n", status);
-    ok(debug_flags == TRUE|| broken(status == STATUS_INVALID_INFO_CLASS) /* NT4 */, "Expected flag TRUE, got %x.\n", debug_flags);
+    ok(!status, "NtQueryInformationProcess failed, status %#x.\n", status);
+    ok(debug_flags == TRUE, "Expected flag TRUE, got %x.\n", debug_flags);
 
-    status = pNtQueryInformationProcess(pi.hProcess, ProcessDebugFlags,
-            &debug_flags, sizeof(debug_flags), NULL);
-    ok(!status || broken(status == STATUS_INVALID_INFO_CLASS) /* NT4 */, "NtQueryInformationProcess failed, status %#x.\n", status);
-    ok(debug_flags == FALSE || broken(status == STATUS_INVALID_INFO_CLASS) /* NT4 */, "Expected flag FALSE, got %x.\n", debug_flags);
-
-    for (;;)
+    for (i = 0; i < sizeof(test_flags)/sizeof(test_flags[0]); i++)
     {
-        DEBUG_EVENT ev;
+        DWORD expected_flags = !(test_flags[i] & DEBUG_ONLY_THIS_PROCESS);
+        sprintf(cmdline, "%s %s %s", argv[0], argv[1], "debuggee");
 
-        ret = WaitForDebugEvent(&ev, INFINITE);
-        ok(ret, "WaitForDebugEvent failed, last error %#x.\n", GetLastError());
-        if (!ret) break;
+        si.cb = sizeof(si);
+        ret = CreateProcessA(NULL, cmdline, NULL, NULL, FALSE, test_flags[i], NULL, NULL, &si, &pi);
+        ok(ret, "CreateProcess failed, last error %#x.\n", GetLastError());
 
-        if (ev.dwDebugEventCode == EXIT_PROCESS_DEBUG_EVENT) break;
+        if (!(test_flags[i] & (DEBUG_PROCESS | DEBUG_ONLY_THIS_PROCESS)))
+        {
+            /* test ProcessDebugFlags before attaching with debugger */
+            status = pNtQueryInformationProcess(pi.hProcess, ProcessDebugFlags,
+                    &debug_flags, sizeof(debug_flags), NULL);
+            ok(!status, "NtQueryInformationProcess failed, status %#x.\n", status);
+            ok(debug_flags == TRUE, "Expected flag TRUE, got %x.\n", debug_flags);
 
-        ret = ContinueDebugEvent(ev.dwProcessId, ev.dwThreadId, DBG_CONTINUE);
-        ok(ret, "ContinueDebugEvent failed, last error %#x.\n", GetLastError());
-        if (!ret) break;
+            ret = DebugActiveProcess(pi.dwProcessId);
+            ok(ret, "DebugActiveProcess failed, last error %#x.\n", GetLastError());
+            expected_flags = FALSE;
+        }
+
+        /* test ProcessDebugFlags after attaching with debugger */
+        status = pNtQueryInformationProcess(pi.hProcess, ProcessDebugFlags,
+                &debug_flags, sizeof(debug_flags), NULL);
+        ok(!status, "NtQueryInformationProcess failed, status %#x.\n", status);
+        ok(debug_flags == expected_flags, "Expected flag %x, got %x.\n", expected_flags, debug_flags);
+
+        if (!(test_flags[i] & CREATE_SUSPENDED))
+        {
+            /* Continue a couple of times to make sure the process is fully initialized,
+             * otherwise Windows XP deadlocks in the following DebugActiveProcess(). */
+            for (;;)
+            {
+                ret = WaitForDebugEvent(&ev, 1000);
+                ok(ret, "WaitForDebugEvent failed, last error %#x.\n", GetLastError());
+                if (!ret) break;
+
+                if (ev.dwDebugEventCode == LOAD_DLL_DEBUG_EVENT) break;
+
+                ret = ContinueDebugEvent(ev.dwProcessId, ev.dwThreadId, DBG_CONTINUE);
+                ok(ret, "ContinueDebugEvent failed, last error %#x.\n", GetLastError());
+                if (!ret) break;
+            }
+
+            result = SuspendThread(pi.hThread);
+            ok(result == 0, "Expected 0, got %u.\n", result);
+        }
+
+        ret = DebugActiveProcessStop(pi.dwProcessId);
+        ok(ret, "DebugActiveProcessStop failed, last error %#x.\n", GetLastError());
+
+        /* test ProcessDebugFlags after detaching debugger */
+        status = pNtQueryInformationProcess(pi.hProcess, ProcessDebugFlags,
+                &debug_flags, sizeof(debug_flags), NULL);
+        ok(!status, "NtQueryInformationProcess failed, status %#x.\n", status);
+        ok(debug_flags == expected_flags, "Expected flag %x, got %x.\n", expected_flags, debug_flags);
+
+        ret = DebugActiveProcess(pi.dwProcessId);
+        ok(ret, "DebugActiveProcess failed, last error %#x.\n", GetLastError());
+
+        /* test ProcessDebugFlags after re-attaching debugger */
+        status = pNtQueryInformationProcess(pi.hProcess, ProcessDebugFlags,
+                &debug_flags, sizeof(debug_flags), NULL);
+        ok(!status, "NtQueryInformationProcess failed, status %#x.\n", status);
+        ok(debug_flags == FALSE, "Expected flag FALSE, got %x.\n", debug_flags);
+
+        result = ResumeThread(pi.hThread);
+        todo_wine ok(result == 2, "Expected 2, got %u.\n", result);
+
+        /* Wait until the process is terminated. On Windows XP the process randomly
+         * gets stuck in a non-continuable exception, so stop after 100 iterations.
+         * On Windows 2003, the debugged process disappears (or stops?) without
+         * any EXIT_PROCESS_DEBUG_EVENT after a couple of events. */
+        for (j = 0; j < 100; j++)
+        {
+            ret = WaitForDebugEvent(&ev, 1000);
+            ok(ret || broken(GetLastError() == ERROR_SEM_TIMEOUT),
+                "WaitForDebugEvent failed, last error %#x.\n", GetLastError());
+            if (!ret) break;
+
+            if (ev.dwDebugEventCode == EXIT_PROCESS_DEBUG_EVENT) break;
+
+            ret = ContinueDebugEvent(ev.dwProcessId, ev.dwThreadId, DBG_CONTINUE);
+            ok(ret, "ContinueDebugEvent failed, last error %#x.\n", GetLastError());
+            if (!ret) break;
+        }
+        ok(j < 100 || broken(j >= 100) /* Win XP */, "Expected less than 100 debug events.\n");
+
+        /* test ProcessDebugFlags after process has terminated */
+        status = pNtQueryInformationProcess(pi.hProcess, ProcessDebugFlags,
+                &debug_flags, sizeof(debug_flags), NULL);
+        ok(!status, "NtQueryInformationProcess failed, status %#x.\n", status);
+        ok(debug_flags == FALSE, "Expected flag FALSE, got %x.\n", debug_flags);
+
+        ret = CloseHandle(pi.hThread);
+        ok(ret, "CloseHandle failed, last error %#x.\n", GetLastError());
+        ret = CloseHandle(pi.hProcess);
+        ok(ret, "CloseHandle failed, last error %#x.\n", GetLastError());
     }
-
-    ret = CloseHandle(pi.hThread);
-    ok(ret, "CloseHandle failed, last error %#x.\n", GetLastError());
-    ret = CloseHandle(pi.hProcess);
-    ok(ret, "CloseHandle failed, last error %#x.\n", GetLastError());
 }
 
 static void test_readvirtualmemory(void)
@@ -1442,6 +1552,8 @@ static void test_mapprotection(void)
     *(unsigned char*)addr = 0xc3;       /* lret ... in both i386 and x86_64 */
 #elif defined(__arm__)
     *(unsigned long*)addr = 0xe12fff1e; /* bx lr */
+#elif defined(__aarch64__)
+    *(unsigned long*)addr = 0xd65f03c0; /* ret */
 #else
     ok(0, "Add a return opcode for your architecture or expect a crash in this test\n");
 #endif
