@@ -633,6 +633,7 @@ static HRESULT WINAPI XA2SRC_FlushSourceBuffers(IXAudio2SourceVoice *iface)
     }
 
     This->nbufs -= to_flush;
+    This->cur_buf = (This->first_buf + This->nbufs) % XAUDIO2_MAX_QUEUED_BUFFERS;
 
     LeaveCriticalSection(&This->lock);
 
@@ -1688,7 +1689,13 @@ static HRESULT WINAPI IXAudio2Impl_CreateMasteringVoice(IXAudio2 *iface,
         goto exit;
     }
 
-    IAudioClient_Start(This->aclient);
+    hr = IAudioClient_Start(This->aclient);
+    if (FAILED(hr))
+    {
+        WARN("Start(IAudioClient) failed: %08x\n", hr);
+        hr = COMPAT_E_DEVICE_INVALIDATED(This->version);
+        goto exit;
+    }
 
     if(This->version <= 20)
         *ppMasteringVoice = (IXAudio2MasteringVoice*)&This->IXAudio20MasteringVoice_iface;
@@ -1800,7 +1807,7 @@ struct xaudio2_cf {
     DWORD version;
 };
 
-struct xaudio2_cf *impl_from_IClassFactory(IClassFactory *iface)
+static struct xaudio2_cf *impl_from_IClassFactory(IClassFactory *iface)
 {
     return CONTAINING_RECORD(iface, struct xaudio2_cf, IClassFactory_iface);
 }
@@ -2090,7 +2097,7 @@ HRESULT WINAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, void **ppv)
     return IClassFactory_QueryInterface(factory, riid, ppv);
 }
 
-/* returns TRUE if there is more data avilable in the buffer, FALSE if the
+/* returns TRUE if there is more data available in the buffer, FALSE if the
  * buffer's data has all been queued */
 static BOOL xa2buffer_queue_period(XA2SourceImpl *src, XA2Buffer *buf, ALuint al_buf)
 {
